@@ -7,11 +7,12 @@
 (put 'bbdb-mua-specific-gnus 'custom-loads '("gnus-bbdb"))
 (put 'nnmail 'custom-loads '("nnmail"))
 (put 'gnus-fun 'custom-loads '("gnus-fun"))
-(put 'gnus-article-emphasis 'custom-loads '("gnus-art"))
+(put 'gnus-article-emphasis 'custom-loads '("assistant" "gnus-art"))
 (put 'gnus-article-headers 'custom-loads '("gnus-art" "gnus-fun" "gnus-sum"))
 (put 'gnus-newsrc 'custom-loads '("gnus-start"))
 (put 'nnmail-procmail 'custom-loads '("nnmail"))
 (put 'gnus-score-kill 'custom-loads '("gnus-kill" "gnus"))
+(put 'spam-crm114 'custom-loads '("spam"))
 (put 'gnus-visual 'custom-loads '("earcon" "gnus-art" "gnus-audio" "gnus" "smiley"))
 (put 'dig 'custom-loads '("dig"))
 (put 'gnus-score-expire 'custom-loads '("gnus-kill" "gnus-score"))
@@ -290,10 +291,18 @@
 (custom-put-if-not 'mailcap 'custom-version "21.1")
 (custom-put-if-not 'mailcap 'group-documentation "Definition of viewers for MIME types.")
 
+
+;;;### (autoloads (assistant) "assistant" "assistant.el" (16575 41047))
+;;; Generated autoloads from assistant.el
+
+(autoload (quote assistant) "assistant" "\
+Assist setting up Emacs based on FILE." t nil)
+
+;;;***
 
 ;;;### (autoloads (binhex-decode-region binhex-decode-region-external
-;;;;;;  binhex-decode-region-internal) "binhex" "binhex.el" (16532
-;;;;;;  16366))
+;;;;;;  binhex-decode-region-internal) "binhex" "binhex.el" (16454
+;;;;;;  52567))
 ;;; Generated autoloads from binhex.el
 
 (defconst binhex-begin-line "^:...............................................................$")
@@ -311,7 +320,7 @@ Binhex decode region between START and END." t nil)
 ;;;***
 
 ;;;### (autoloads (canlock-verify canlock-insert-header) "canlock"
-;;;;;;  "canlock.el" (16532 16368))
+;;;;;;  "canlock.el" (16454 52567))
 ;;; Generated autoloads from canlock.el
 
 (autoload (quote canlock-insert-header) "canlock" "\
@@ -328,7 +337,7 @@ it fails." t nil)
 ;;;;;;  gnus-article-outlook-repair-attribution gnus-article-outlook-unwrap-lines
 ;;;;;;  gnus-outlook-display-hook gnus-outlook-deuglify-unwrap-max
 ;;;;;;  gnus-outlook-deuglify-unwrap-min) "deuglify" "deuglify.el"
-;;;;;;  (16532 16368))
+;;;;;;  (16429 29965))
 ;;; Generated autoloads from deuglify.el
 
 (defvar gnus-outlook-deuglify-unwrap-min 45 "\
@@ -372,7 +381,7 @@ Play sounds in message buffers." t nil)
 ;;;***
 
 ;;;### (autoloads (fill-flowed fill-flowed-encode) "flow-fill" "flow-fill.el"
-;;;;;;  (16532 16369))
+;;;;;;  (16446 32843))
 ;;; Generated autoloads from flow-fill.el
 
 (autoload (quote fill-flowed-encode) "flow-fill" nil nil nil)
@@ -382,7 +391,7 @@ Play sounds in message buffers." t nil)
 ;;;***
 
 ;;;### (autoloads (gnus gnus-other-frame gnus-slave gnus-no-server
-;;;;;;  gnus-slave-no-server) "gnus" "gnus.el" (16532 16394))
+;;;;;;  gnus-slave-no-server) "gnus" "gnus.el" (16580 59670))
 ;;; Generated autoloads from gnus.el
 
 (autoload (quote gnus-slave-no-server) "gnus" "\
@@ -421,7 +430,7 @@ prompt the user for the name of an NNTP server to use." t nil)
 ;;;### (autoloads (gnus-agent-regenerate gnus-agent-batch gnus-agent-batch-fetch
 ;;;;;;  gnus-agent-delete-group gnus-agent-rename-group gnus-agent-possibly-save-gcc
 ;;;;;;  gnus-agentize gnus-slave-unplugged gnus-plugged gnus-unplugged)
-;;;;;;  "gnus-agent" "gnus-agent.el" (16532 16369))
+;;;;;;  "gnus-agent" "gnus-agent.el" (16551 30130))
 ;;; Generated autoloads from gnus-agent.el
 
 (autoload (quote gnus-unplugged) "gnus-agent" "\
@@ -470,7 +479,7 @@ If CLEAN, obsolete (ignore)." t nil)
 ;;;***
 
 ;;;### (autoloads (gnus-article-prepare-display) "gnus-art" "gnus-art.el"
-;;;;;;  (16532 16370))
+;;;;;;  (16582 17140))
 ;;; Generated autoloads from gnus-art.el
 
 (autoload (quote gnus-article-prepare-display) "gnus-art" "\
@@ -546,7 +555,7 @@ Call this function to hook BBDB into message-mode." nil nil)
 
 ;;;### (autoloads (gnus-cache-delete-group gnus-cache-rename-group
 ;;;;;;  gnus-cache-generate-nov-databases gnus-cache-generate-active
-;;;;;;  gnus-jog-cache) "gnus-cache" "gnus-cache.el" (16532 16374))
+;;;;;;  gnus-jog-cache) "gnus-cache" "gnus-cache.el" (16555 57221))
 ;;; Generated autoloads from gnus-cache.el
 
 (autoload (quote gnus-jog-cache) "gnus-cache" "\
@@ -574,7 +583,7 @@ next enabled. Depends upon the caller to determine whether group deletion is sup
 ;;;***
 
 ;;;### (autoloads (gnus-delay-initialize gnus-delay-send-queue gnus-delay-article)
-;;;;;;  "gnus-delay" "gnus-delay.el" (16532 16375))
+;;;;;;  "gnus-delay" "gnus-delay.el" (16383 51495))
 ;;; Generated autoloads from gnus-delay.el
 
 (defgroup gnus-delay nil "Arrange for sending postings later." :group (quote gnus))
@@ -606,7 +615,7 @@ Checking delayed messages is skipped if optional arg NO-CHECK is non-nil." nil n
 ;;;***
 
 ;;;### (autoloads (turn-on-gnus-dired-mode) "gnus-dired" "gnus-dired.el"
-;;;;;;  (16532 16375))
+;;;;;;  (16378 35690))
 ;;; Generated autoloads from gnus-dired.el
 
 (autoload (quote turn-on-gnus-dired-mode) "gnus-dired" "\
@@ -615,7 +624,7 @@ Convenience method to turn on gnus-dired-mode." nil nil)
 ;;;***
 
 ;;;### (autoloads (gnus-draft-reminder) "gnus-draft" "gnus-draft.el"
-;;;;;;  (16532 16375))
+;;;;;;  (16555 57221))
 ;;; Generated autoloads from gnus-draft.el
 
 (autoload (quote gnus-draft-reminder) "gnus-draft" "\
@@ -625,7 +634,7 @@ Reminder user if there are unsent drafts." t nil)
 
 ;;;### (autoloads (gnus-convert-png-to-face gnus-convert-face-to-png
 ;;;;;;  gnus-face-from-file gnus-x-face-from-file gnus-insert-random-x-face-header
-;;;;;;  gnus-random-x-face) "gnus-fun" "gnus-fun.el" (16532 16375))
+;;;;;;  gnus-random-x-face) "gnus-fun" "gnus-fun.el" (16427 9214))
 ;;; Generated autoloads from gnus-fun.el
 
 (autoload (quote gnus-random-x-face) "gnus-fun" "\
@@ -652,7 +661,7 @@ FILE should be a PNG file that's 48x48 and smaller than or equal to
 ;;;***
 
 ;;;### (autoloads (gnus-fetch-group-other-frame gnus-fetch-group)
-;;;;;;  "gnus-group" "gnus-group.el" (16532 16376))
+;;;;;;  "gnus-group" "gnus-group.el" (16570 22580))
 ;;; Generated autoloads from gnus-group.el
 
 (autoload (quote gnus-fetch-group) "gnus-group" "\
@@ -664,8 +673,8 @@ Pop up a frame and enter GROUP." t nil)
 
 ;;;***
 
-;;;### (autoloads (gnus-batch-score) "gnus-kill" "gnus-kill.el" (16532
-;;;;;;  16376))
+;;;### (autoloads (gnus-batch-score) "gnus-kill" "gnus-kill.el" (16456
+;;;;;;  23589))
 ;;; Generated autoloads from gnus-kill.el
 
 (defalias (quote gnus-batch-kill) (quote gnus-batch-score))
@@ -677,8 +686,8 @@ Usage: emacs -batch -l ~/.emacs -l gnus -f gnus-batch-score" t nil)
 ;;;***
 
 ;;;### (autoloads (gnus-mailing-list-mode gnus-mailing-list-insinuate
-;;;;;;  turn-on-gnus-mailing-list-mode) "gnus-ml" "gnus-ml.el" (16532
-;;;;;;  16376))
+;;;;;;  turn-on-gnus-mailing-list-mode) "gnus-ml" "gnus-ml.el" (16446
+;;;;;;  47459))
 ;;; Generated autoloads from gnus-ml.el
 
 (autoload (quote turn-on-gnus-mailing-list-mode) "gnus-ml" nil nil nil)
@@ -695,8 +704,8 @@ Minor mode for providing mailing-list commands.
 ;;;***
 
 ;;;### (autoloads (gnus-group-split-fancy gnus-group-split gnus-group-split-update
-;;;;;;  gnus-group-split-setup) "gnus-mlspl" "gnus-mlspl.el" (16532
-;;;;;;  16376))
+;;;;;;  gnus-group-split-setup) "gnus-mlspl" "gnus-mlspl.el" (16446
+;;;;;;  47459))
 ;;; Generated autoloads from gnus-mlspl.el
 
 (autoload (quote gnus-group-split-setup) "gnus-mlspl" "\
@@ -788,7 +797,7 @@ Calling (gnus-group-split-fancy nil nil \"mail.others\") returns:
 ;;;***
 
 ;;;### (autoloads (gnus-change-server) "gnus-move" "gnus-move.el"
-;;;;;;  (16532 16376))
+;;;;;;  (16383 51495))
 ;;; Generated autoloads from gnus-move.el
 
 (autoload (quote gnus-change-server) "gnus-move" "\
@@ -798,7 +807,7 @@ Update the .newsrc.eld file to reflect the change of nntp server." t nil)
 ;;;***
 
 ;;;### (autoloads (gnus-button-reply gnus-button-mailto gnus-msg-mail)
-;;;;;;  "gnus-msg" "gnus-msg.el" (16532 16376))
+;;;;;;  "gnus-msg" "gnus-msg.el" (16555 65225))
 ;;; Generated autoloads from gnus-msg.el
 
 (autoload (quote gnus-msg-mail) "gnus-msg" "\
@@ -818,8 +827,8 @@ Like `message-reply'." t nil)
 
 ;;;### (autoloads (gnus-namazu-insinuate gnus-namazu-stop-update
 ;;;;;;  gnus-namazu-update-all-indices gnus-namazu-update-index gnus-namazu-create-index
-;;;;;;  gnus-namazu-search) "gnus-namazu" "gnus-namazu.el" (16532
-;;;;;;  16377))
+;;;;;;  gnus-namazu-search) "gnus-namazu" "gnus-namazu.el" (16547
+;;;;;;  8132))
 ;;; Generated autoloads from gnus-namazu.el
 
 (autoload (quote gnus-namazu-search) "gnus-namazu" "\
@@ -843,8 +852,8 @@ Stop the running indexer of Namazu." t nil)
 ;;;***
 
 ;;;### (autoloads (gnus-treat-newsgroups-picon gnus-treat-mail-picon
-;;;;;;  gnus-treat-from-picon) "gnus-picon" "gnus-picon.el" (16532
-;;;;;;  16378))
+;;;;;;  gnus-treat-from-picon) "gnus-picon" "gnus-picon.el" (16554
+;;;;;;  35505))
 ;;; Generated autoloads from gnus-picon.el
 
 (autoload (quote gnus-treat-from-picon) "gnus-picon" "\
@@ -865,7 +874,7 @@ If picons are already displayed, remove them." t nil)
 ;;;;;;  gnus-sorted-nintersection gnus-sorted-range-intersection
 ;;;;;;  gnus-sorted-intersection gnus-intersection gnus-sorted-complement
 ;;;;;;  gnus-sorted-ndifference gnus-sorted-difference) "gnus-range"
-;;;;;;  "gnus-range.el" (16532 16379))
+;;;;;;  "gnus-range.el" (16453 29591))
 ;;; Generated autoloads from gnus-range.el
 
 (autoload (quote gnus-sorted-difference) "gnus-range" "\
@@ -912,7 +921,7 @@ Add NUM into sorted LIST by side effect." nil nil)
 ;;;***
 
 ;;;### (autoloads (gnus-registry-install-hooks gnus-registry-initialize)
-;;;;;;  "gnus-registry" "gnus-registry.el" (16532 16379))
+;;;;;;  "gnus-registry" "gnus-registry.el" (16561 7460))
 ;;; Generated autoloads from gnus-registry.el
 
 (autoload (quote gnus-registry-initialize) "gnus-registry" nil t nil)
@@ -944,7 +953,7 @@ See the documentation for these variables and functions for details." t nil)
 ;;;***
 
 ;;;### (autoloads (gnus-batch-brew-soup) "gnus-soup" "gnus-soup.el"
-;;;;;;  (16532 16380))
+;;;;;;  (16383 51496))
 ;;; Generated autoloads from gnus-soup.el
 
 (autoload (quote gnus-batch-brew-soup) "gnus-soup" "\
@@ -962,7 +971,7 @@ Note -- this function hasn't been implemented yet." t nil)
 ;;;***
 
 ;;;### (autoloads (gnus-update-format) "gnus-spec" "gnus-spec.el"
-;;;;;;  (16532 16380))
+;;;;;;  (16406 21783))
 ;;; Generated autoloads from gnus-spec.el
 
 (autoload (quote gnus-update-format) "gnus-spec" "\
@@ -971,8 +980,8 @@ Update the format specification near point." t nil)
 ;;;***
 
 ;;;### (autoloads (gnus-fixup-nnimap-unread-after-getting-new-news
-;;;;;;  gnus-declare-backend) "gnus-start" "gnus-start.el" (16532
-;;;;;;  16381))
+;;;;;;  gnus-declare-backend) "gnus-start" "gnus-start.el" (16570
+;;;;;;  22581))
 ;;; Generated autoloads from gnus-start.el
 
 (autoload (quote gnus-declare-backend) "gnus-start" "\
@@ -991,7 +1000,7 @@ Add the window configuration CONF to `gnus-buffer-configuration'." nil nil)
 
 ;;;***
 
-;;;### (autoloads (html2text) "html2text" "html2text.el" (16532 16396))
+;;;### (autoloads (html2text) "html2text" "html2text.el" (16528 14600))
 ;;; Generated autoloads from html2text.el
 
 (autoload (quote html2text) "html2text" "\
@@ -1015,8 +1024,8 @@ Convert HTML to plain text in the current buffer." t nil)
 ;;;;;;  message-signature-separator message-from-style message-cross-post-note-function
 ;;;;;;  message-followup-to-note message-cross-post-note message-cross-post-default
 ;;;;;;  message-archive-note message-archive-header message-mark-insert-end
-;;;;;;  message-mark-insert-begin) "message" "message.el" (16532
-;;;;;;  16397))
+;;;;;;  message-mark-insert-begin) "message" "message.el" (16582
+;;;;;;  64747))
 ;;; Generated autoloads from message.el
 
 (defvar message-mark-insert-begin "--8<---------------cut here---------------start------------->8---\n" "\
@@ -1304,7 +1313,7 @@ If NO-DISPLAY is nil, display it. Otherwise, do nothing after replacing." nil ni
 ;;;***
 
 ;;;### (autoloads (mm-url-insert-file-contents-external mm-url-insert-file-contents)
-;;;;;;  "mm-url" "mm-url.el" (16532 16403))
+;;;;;;  "mm-url" "mm-url.el" (16454 52570))
 ;;; Generated autoloads from mm-url.el
 
 (autoload (quote mm-url-insert-file-contents) "mm-url" "\
@@ -1336,7 +1345,7 @@ Dissect the current buffer and return a list of uu handles." nil nil)
 
 ;;;### (autoloads (mml2015-self-encrypt mml2015-sign mml2015-encrypt
 ;;;;;;  mml2015-verify-test mml2015-verify mml2015-decrypt-test mml2015-decrypt)
-;;;;;;  "mml2015" "mml2015.el" (16532 16406))
+;;;;;;  "mml2015" "mml2015.el" (16501 59509))
 ;;; Generated autoloads from mml2015.el
 
 (autoload (quote mml2015-decrypt) "mml2015" nil nil nil)
@@ -1356,7 +1365,7 @@ Dissect the current buffer and return a list of uu handles." nil nil)
 ;;;***
 
 ;;;### (autoloads (nndiary-generate-nov-databases) "nndiary" "nndiary.el"
-;;;;;;  (16532 16407))
+;;;;;;  (16378 35721))
 ;;; Generated autoloads from nndiary.el
 
 (autoload (quote nndiary-generate-nov-databases) "nndiary" "\
@@ -1364,7 +1373,7 @@ Generate NOV databases in all nndiary directories." t nil)
 
 ;;;***
 
-;;;### (autoloads (nndoc-add-type) "nndoc" "nndoc.el" (16532 16407))
+;;;### (autoloads (nndoc-add-type) "nndoc" "nndoc.el" (16481 7404))
 ;;; Generated autoloads from nndoc.el
 
 (autoload (quote nndoc-add-type) "nndoc" "\
@@ -1377,7 +1386,7 @@ symbol in the alist." nil nil)
 ;;;***
 
 ;;;### (autoloads (nnfolder-generate-active-file) "nnfolder" "nnfolder.el"
-;;;;;;  (16532 16408))
+;;;;;;  (16524 14588))
 ;;; Generated autoloads from nnfolder.el
 
 (autoload (quote nnfolder-generate-active-file) "nnfolder" "\
@@ -1387,7 +1396,7 @@ This command does not work if you use short group names." t nil)
 ;;;***
 
 ;;;### (autoloads (nnkiboze-generate-groups) "nnkiboze" "nnkiboze.el"
-;;;;;;  (16532 16411))
+;;;;;;  (16383 51498))
 ;;; Generated autoloads from nnkiboze.el
 
 (autoload (quote nnkiboze-generate-groups) "nnkiboze" "\
@@ -1397,7 +1406,7 @@ Finds out what articles are to be part of the nnkiboze groups." t nil)
 ;;;***
 
 ;;;### (autoloads (nnml-generate-nov-databases) "nnml" "nnml.el"
-;;;;;;  (16532 16413))
+;;;;;;  (16524 14589))
 ;;; Generated autoloads from nnml.el
 
 (autoload (quote nnml-generate-nov-databases) "nnml" "\
@@ -1439,7 +1448,7 @@ them into characters should be done separately." t nil)
 ;;;***
 
 ;;;### (autoloads (gnus-score-mode) "score-mode" "score-mode.el"
-;;;;;;  (16532 16420))
+;;;;;;  (16377 15531))
 ;;; Generated autoloads from score-mode.el
 
 (autoload (quote gnus-score-mode) "score-mode" "\
@@ -1451,7 +1460,7 @@ This mode is an extended emacs-lisp mode.
 ;;;***
 
 ;;;### (autoloads (sieve-upload-and-bury sieve-upload sieve-manage)
-;;;;;;  "sieve" "sieve.el" (16532 16421))
+;;;;;;  "sieve" "sieve.el" (16416 7549))
 ;;; Generated autoloads from sieve.el
 
 (autoload (quote sieve-manage) "sieve" nil t nil)
@@ -1462,8 +1471,8 @@ This mode is an extended emacs-lisp mode.
 
 ;;;***
 
-;;;### (autoloads (sieve-mode) "sieve-mode" "sieve-mode.el" (16532
-;;;;;;  16421))
+;;;### (autoloads (sieve-mode) "sieve-mode" "sieve-mode.el" (16454
+;;;;;;  52570))
 ;;; Generated autoloads from sieve-mode.el
 
 (autoload (quote sieve-mode) "sieve-mode" "\
@@ -1486,7 +1495,7 @@ Turning on Sieve mode runs `sieve-mode-hook'." t nil)
 
 ;;;***
 
-;;;### (autoloads (spam-initialize) "spam" "spam.el" (16532 16422))
+;;;### (autoloads (spam-initialize) "spam" "spam.el" (16580 59670))
 ;;; Generated autoloads from spam.el
 
 (autoload (quote spam-initialize) "spam" "\
@@ -1500,7 +1509,7 @@ installed through spam-necessary-extra-headers." t nil)
 
 ;;;### (autoloads (spam-report-deagentize spam-report-agentize spam-report-url-to-file
 ;;;;;;  spam-report-url-ping-mm-url spam-report-process-queue) "spam-report"
-;;;;;;  "spam-report.el" (16532 16422))
+;;;;;;  "spam-report.el" (16580 59670))
 ;;; Generated autoloads from spam-report.el
 
 (autoload (quote spam-report-process-queue) "spam-report" "\
@@ -1590,7 +1599,7 @@ If DATE is malformed, return a time value of zeros." nil nil)
 
 ;;;### (autoloads (uudecode-decode-region uudecode-decode-region-internal
 ;;;;;;  uudecode-decode-region-external) "uudecode" "uudecode.el"
-;;;;;;  (16532 16423))
+;;;;;;  (16454 52570))
 ;;; Generated autoloads from uudecode.el
 
 (autoload (quote uudecode-decode-region-external) "uudecode" "\
